@@ -27,7 +27,6 @@
 
 #include <glib.h>
 #include <libgweather/gweather-timezone.h>
-#include <libgweather/weather.h>
 
 G_BEGIN_DECLS
 
@@ -80,7 +79,7 @@ void                   gweather_location_free_timezones (GWeatherLocation  *loc,
 const char            *gweather_location_get_code       (GWeatherLocation  *loc);
 char                  *gweather_location_get_city_name  (GWeatherLocation  *loc);
 
-WeatherInfo           *gweather_location_get_weather    (GWeatherLocation  *loc);
+GWeatherLocation      *gweather_location_find_by_station_code (const gchar *code);
 
 G_END_DECLS
 
